# -*- python -*-
# $RCSfile: quit.py,v $
# $Revision: 1.31 $
# $Author: reida $
# $Date: 2006/08/23 23:56:06 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from oof2.common import debug
from oof2.common import mainthread
from oof2.common import subthread
from oof2.common.IO import mainmenu
from oof2.common.IO import reporter
import oof2.common.quit
import sys

# There's no code in here that actually addresses the GUI directly.
# This routine should work in non-GUI mode too.  Should it be moved to
# common/quit.py?  The code currently in common/quit.py differs from
# this code only in that common/quit.py doesn't give the user a chance
# to cancel the quitting procedure.  Presumably in non-GUI mode it's
# less easy to quit by mistake.

def quitQuery():
    if not oof2.common.quit.quiet():
        if mainmenu.OOF.logChanged():
            answer = reporter.query("Save log file before quitting?",
                                    "Save", "Don't Save", "Cancel",
                                    default="Don't Save")
            if answer=="Cancel" or answer is None:
                return 0
            if answer=="Save":
                mainmenu.OOF.File.Save.Python_Log.invokeGUICallback()
                mainmenu.cleanlog()
##        else:
##            answer = reporter.query("Quitting so soon?", "Yes", "No",
##                                    default="Yes")
##            if answer == "No":
##                return 0
    return 1

# OOFMenu gui callback for the Quit commands.  Installed by
# mainmenuGUI.py and subWindow.py.
def quit(menuitem):
    if quitQuery():
        gui_cleanup()


####################################################################
#
# Over-ride cleanup, to do fancier things with the lock.
#

# The point of this is to block on a *subthread* while waiting for the
# quit lock, and then once you have it, transfer control back to the
# main thread for the actual cleanup.
def gui_quit_subthread():
    oof2.common.quit.quitlock.acquire()
    try:
        mainthread.run(oof2.common.quit.cleanup, kwargs={"locked": False})
    finally:
        oof2.common.quit.quitlock.release()
    
def gui_cleanup():
    subthread.execute_immortal(gui_quit_subthread)
    
