# -*- python -*-
# $RCSfile: fieldPage.py,v $
# $Revision: 1.49 $
# $Author: langer $
# $Date: 2007/01/08 18:27:31 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 


from oof2.SWIG.common import switchboard
from oof2.common import debug
from oof2.common import guitop
from oof2.common import labeltree
from oof2.common import microstructure
from oof2.common.IO import parameter
from oof2.common.IO import whoville
from oof2.common.IO.GUI import gtklogger
from oof2.common.IO.GUI import oofGUI
from oof2.common.IO.GUI import parameterwidgets
from oof2.common.IO.GUI import whowidget
from oof2.engine import fieldinit
from oof2.engine import skeletoncontext
from oof2.engine.IO import subproblemmenu
import oof2.SWIG.engine.field
import oof2.engine.IO.meshmenu
import oof2.engine.mesh

import gobject
import gtk
import string

allFields = oof2.SWIG.engine.field.allFields

subpmenu = subproblemmenu.subproblemMenu
meshmenu = oof2.engine.IO.meshmenu.meshmenu

class ButtonSignal:
    def __init__(self, button, signal):
        self.button = button
        self.signal = signal
    def set(self, active):
        debug.mainthreadTest()
        self.signal.block()
        self.button.set_active(active)
        self.signal.unblock()
    def set_sensitive(self, sensitivity):
        debug.mainthreadTest()
        #if self.button.get_property('sensitive') != sensitivity:
        self.button.set_sensitive(sensitivity)


class FieldPage(oofGUI.MainPage):
    def __init__(self):
        oofGUI.MainPage.__init__(self, name="Fields", ordering=210,
                                 tip="Define fields on a finite element mesh.")
        mainbox = gtk.VBox(spacing=2)
        self.gtk.add(mainbox)

        align = gtk.Alignment(xalign=0.5)
        mainbox.pack_start(align, expand=0, fill=0)
        centerbox = gtk.HBox(spacing=3)
        align.add(centerbox)
        self.subpwidget = whowidget.WhoWidget(
            oof2.engine.subproblemcontext.subproblems, scope=self)
##            callback=self.subpCB
        switchboard.requestCallbackMain(self.subpwidget, self.subpwidgetCB)
        label = gtk.Label("Microstructure=")
        label.set_alignment(1.0, 0.5)
        centerbox.pack_start(label, expand=0, fill=0)
        centerbox.pack_start(self.subpwidget.gtk[0], expand=0, fill=0)
        label = gtk.Label("Skeleton=")
        label.set_alignment(1.0, 0.5)
        centerbox.pack_start(label, expand=0, fill=0)
        centerbox.pack_start(self.subpwidget.gtk[1], expand=0, fill=0)
        label = gtk.Label("Mesh=")
        label.set_alignment(1.0, 0.5)
        centerbox.pack_start(label, expand=0, fill=0)
        centerbox.pack_start(self.subpwidget.gtk[2], expand=0, fill=0)
        label = gtk.Label("SubProblem=")
        label.set_alignment(1.0, 0.5)
        centerbox.pack_start(label, expand=0, fill=0)
        centerbox.pack_start(self.subpwidget.gtk[3], expand=0, fill=0)

        vpane = gtk.VPaned()
        gtklogger.setWidgetName(vpane, 'Pane')
        mainbox.pack_start(vpane, expand=1, fill=1, padding=3)

        fieldframe = gtk.Frame("Fields")
        fieldframe.set_shadow_type(gtk.SHADOW_IN)
        vpane.pack1(fieldframe, resize=1, shrink=0)
        gtklogger.connect_passive(fieldframe, 'size-allocate')
        scroll = gtk.ScrolledWindow()
        gtklogger.logScrollBars(scroll, "Fields")
        scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        fieldframe.add(scroll)
        bbox = gtk.VBox()
        scroll.add_with_viewport(bbox)
        self.fieldtable = gtk.Table()
        bbox.pack_start(self.fieldtable, expand=0, fill=0)
        self.build_fieldTable()

        frame = gtk.Frame("Field Initialization (for all Subproblems)")
        frame.set_shadow_type(gtk.SHADOW_IN)
        vpane.pack2(frame, resize=1, shrink=0)
        ivbox = gtk.VBox()
        frame.add(ivbox)
        self.initscroll = gtk.ScrolledWindow()
        gtklogger.logScrollBars(self.initscroll, "InitScroll")
        self.initscroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.initscroll.set_shadow_type(gtk.SHADOW_IN)
        ivbox.pack_start(self.initscroll, expand=1, fill=1)

        # The ListStore just contains the defined Fields.  The
        # TreeView displays their names and initializers.
        self.initlist = gtk.ListStore(gobject.TYPE_PYOBJECT)
        self.initview = gtk.TreeView(self.initlist)
        gtklogger.setWidgetName(self.initview, 'Initializers')
        self.initscroll.add(self.initview)
        self.initview.set_headers_clickable(False)
        fieldnamecell = gtk.CellRendererText()
        fieldnamecol = gtk.TreeViewColumn('Field')
        self.initview.append_column(fieldnamecol)
        fieldnamecol.pack_start(fieldnamecell, expand=False)
        fieldnamecol.set_cell_data_func(fieldnamecell, self.renderFieldName)
                                        
        fieldinitcell = gtk.CellRendererText()
        fieldinitcol = gtk.TreeViewColumn('Initializer')
        self.initview.append_column(fieldinitcol)
        fieldinitcol.pack_start(fieldinitcell, expand=True)
        fieldinitcol.set_cell_data_func(fieldinitcell, self.renderFieldInit)

        selection = self.initview.get_selection()
        gtklogger.adoptGObject(selection, self.initview,
                               access_method=self.initview.get_selection)
        self.initselsignal = gtklogger.connect(selection, 'changed',
                                               self.initSelectCB)
        gtklogger.connect(self.initview, 'row-activated',
                          self.initActivateRowCB)
        
        self.initmsg = gtk.Label("Fields defined above\ncan be initialized here\nafter a Mesh is created.")
        ivbox.pack_start(self.initmsg, expand=1, fill=1)

        bbox = gtk.HBox()
        ivbox.pack_start(bbox, expand=0, fill=0, padding=2)

        self.initbutton = gtk.Button('Set Initializer...')
        gtklogger.setWidgetName(self.initbutton, 'Set Init')
        gtklogger.connect(self.initbutton, 'clicked', self.initbuttonCB)
        guitop.top().tooltips.set_tip(self.initbutton,
                                      'Click to initialize the selected field. Alternatively, a double-click on the desired field brings up initializer.')
        bbox.pack_start(self.initbutton, expand=0, fill=0, padding=2)

        self.copyinitbutton = gtk.Button("Copy Initializers...")
        gtklogger.setWidgetName(self.copyinitbutton, 'CopyInit')
        gtklogger.connect(self.copyinitbutton, 'clicked', self.copyinitCB)
        bbox.pack_start(self.copyinitbutton, expand=0, fill=0, padding=2)
        guitop.top().tooltips.set_tip(self.copyinitbutton,
                                      "Copy field initializers from the current mesh to another mesh.")

        self.clearinitbutton = gtk.Button("Clear Initializers")
        gtklogger.setWidgetName(self.clearinitbutton, 'ClearInit')
        gtklogger.connect(self.clearinitbutton, 'clicked', self.clearinitCB)
        bbox.pack_start(self.clearinitbutton, expand=0, fill=0, padding=2)
        guitop.top().tooltips.set_tip(self.clearinitbutton,
                                      "Remove the field initializers from the current mesh.")

        self.copybutton = gtk.Button("Copy Field State...")
        gtklogger.setWidgetName(self.copybutton, 'CopyField')
        gtklogger.connect(self.copybutton, 'clicked', self.copyfstateCB)
        bbox.pack_end(self.copybutton, expand=0, fill=0, padding=2)
        guitop.top().tooltips.set_tip(self.copybutton,
                                      "Copy all field status variables from the current subproblem to another subproblem.")

        self.sensitize()
        switchboard.requestCallbackMain(("new who", "Microstructure"),
                                         self.newMSorSkeletonOrMesh)
        switchboard.requestCallbackMain(("new who", "Skeleton"),
                                        self.newMSorSkeletonOrMesh)
        switchboard.requestCallbackMain(("new who", "Mesh"),
                                        self.newMSorSkeletonOrMesh)
##        switchboard.requestCallbackMain(("new who", "SubProblem"),
##                                        self.newSubProblem)
        switchboard.requestCallbackMain("made reservation",
                                        self.reservationChanged)
        switchboard.requestCallbackMain("cancelled reservation",
                                        self.reservationChanged)
        switchboard.requestCallbackMain("new field", self.newFieldCB)
        switchboard.requestCallbackMain("field defined", self.defineFldCB)
        switchboard.requestCallbackMain("field activated", self.activateFldCB)
        switchboard.requestCallbackMain("field inplane", self.inplaneFldCB)
        switchboard.requestCallbackMain("field initialized", self.initFldCB)

    def show(self):
        debug.mainthreadTest()
        self.gtk.show_all()
        self.update()

    def build_fieldTable(self):
        debug.mainthreadTest()
        self.fieldtable.foreach(gtk.Object.destroy) # clear the table
        self.fieldbuttons = {}
        self.fieldtable.resize(len(allFields), 6)
        self.fieldtable.attach(gtk.VSeparator(), 1,2, 0,len(allFields),
                               xoptions=0, yoptions=gtk.EXPAND|gtk.FILL)
        self.fieldtable.set_col_spacing(0, 3)
        row = 0
        for fld in allFields:
            label = gtk.Label(fld.name())
            label.set_alignment(1.0, 0.5)
            self.fieldtable.attach(label, 0,1, row,row+1,
                                   xoptions=gtk.FILL, yoptions=0)
            button = gtk.CheckButton('defined')
            gtklogger.setWidgetName(button, fld.name()+" defined")
            signal = gtklogger.connect(button, 'clicked',
                                       self.fieldDefineCB, fld)
            self.fieldbuttons[(fld.name(), "defined")] = ButtonSignal(button,
                                                                      signal)
            self.fieldtable.attach(button, 2,3, row, row+1,
                                   xoptions=0, yoptions=0)
            self.setFieldDefineTip(button, fld)

            button = gtk.CheckButton('active')
            gtklogger.setWidgetName(button, fld.name()+" active")
            signal = gtklogger.connect(button, 'clicked',
                                       self.fieldActiveCB, fld)
            self.fieldbuttons[(fld.name(), "active")] = ButtonSignal(button,
                                                                     signal)
            self.fieldtable.attach(button, 3,4, row, row+1,
                                   xoptions=0, yoptions=0)
            self.setFieldActiveTip(button, fld)

            button = gtk.CheckButton('in-plane')
            gtklogger.setWidgetName(button, fld.name() + " in-plane")
            signal = gtklogger.connect(button, 'clicked',
                                       self.fieldInPlaneCB, fld)
            self.fieldbuttons[(fld.name(), "inplane")] = ButtonSignal(button,
                                                                      signal)
            self.fieldtable.attach(button, 4,5, row, row+1,
                                   xoptions=0, yoptions=0)
            self.setFieldInPlaneTip(button, fld)

            row += 1

    def newFieldCB(self):               # switchboard "new field"
        self.build_fieldTable()
        self.show()

    def fieldDefineCB(self, button, field): # gtk callback
        if button.get_active():
            subpmenu.Field.Define(subproblem=self.currentFullSubProblemName(),
                                  field=field)
        else:
            subpmenu.Field.Undefine(subproblem=self.currentFullSubProblemName(),
                                    field=field)
        self.setFieldDefineTip(button, field)

    def setFieldDefineTip(self, button, field):
        if button.get_active():
            verb = "Undefine"
        else:
            verb = "Define"
        guitop.top().tooltip(button, "%s the %s field on the mesh.  Only defined fields have values." % (verb, field.name()))

    def fieldActiveCB(self, button, field): # gtk callback
        if button.get_active():
            subpmenu.Field.Activate(
                subproblem=self.currentFullSubProblemName(), field=field)
        else:
            subpmenu.Field.Deactivate(
                subproblem=self.currentFullSubProblemName(), field=field)
        self.setFieldActiveTip(button, field)
        
    def setFieldActiveTip(self, button, field):
        if button.get_active():
            verb = "Deactivate"
        else:
            verb = "Activate"
        guitop.top().tooltip(button, "%s the %s field on the subproblem. The solver finds the values of the active fields by solving the active equations." % (verb, field.name()))

    def fieldInPlaneCB(self, button, field): # gtk callback
        debug.mainthreadTest()
        if button.get_active():
            meshmenu.Field.In_Plane(
                mesh=self.currentFullMeshName(),
                field = field)
        else:
            meshmenu.Field.Out_of_Plane(
                mesh=self.currentFullMeshName(),
                field = field)
        self.setFieldInPlaneTip(button, field)
        
    def setFieldInPlaneTip(self, button, field):
        debug.mainthreadTest()
        if button.get_active():
            verb = "Do not constrain"
        else:
            verb = "Constrain"
        guitop.top().tooltip(button, "%s the derivatives of the %s field to lie in the x-y plane."
                             % (verb, field.name()))

    def defineFldCB(self, subpname, fieldname, defined): # sb "field defined"
        if subpname == self.currentFullSubProblemName():
            self.initselsignal.block()
            try:
                self.fieldbuttons[(fieldname, "defined")].set(defined)
                self.update()
            finally:
                self.initselsignal.unblock()

    def activateFldCB(self, subpname, fieldname, active): # sb "field activated"
        if subpname == self.currentFullSubProblemName():
            self.fieldbuttons[(fieldname, "active")].set(active)

    def inplaneFldCB(self, meshname, fieldname, inplane): # sb "field inplane"
        if meshname == self.currentFullMeshName():
            self.fieldbuttons[(fieldname, "inplane")].set(inplane)

    ########################

    def currentFullMeshName(self):
        return self.subpwidget.get_value(depth=3)
    def currentMeshContext(self):
        try:
            return oof2.engine.mesh.meshes[self.currentFullMeshName()]
        except KeyError:
            return None
    def currentMesh(self):
        meshctxt = self.currentMeshContext()
        if meshctxt:
            return meshctxt.getObject()
            
    def currentFullSubProblemName(self):
        return self.subpwidget.get_value()
    def currentSubProblemContext(self):
        try:
            return oof2.engine.subproblemcontext.subproblems[
                self.currentFullSubProblemName()]
        except KeyError:
            return None
    def currentSubProblem(self):
        subpc = self.currentSubProblemContext()
        if subpc:
            return subpc.getObject()

    def sensitize(self):
        debug.mainthreadTest()
        subpctxt = self.currentSubProblemContext()
        subpok = subpctxt is not None and not subpctxt.query_reservation()
        meshctxt = self.currentMeshContext()
        meshok = meshctxt is not None
        meshavailable = meshok and not meshctxt.query_reservation()
        self.fieldtable.set_sensitive(subpok)
        self.initview.set_sensitive(subpok)
        self.initbutton.set_sensitive(meshavailable and
                                      self.selectedField() is not None)
        self.copyinitbutton.set_sensitive(meshok and
                                          len(meshctxt.initializers) > 0)
        self.clearinitbutton.set_sensitive(meshavailable and
                                           len(meshctxt.initializers) > 0)
        self.copybutton.set_sensitive(subpok)
        
    def update(self):
        self.set_state(self.currentFullSubProblemName())

    def set_state(self, subppath):
        debug.mainthreadTest()
        path = labeltree.makePath(subppath)
        self.subpwidget.set_value(path)
        # Retrieve the subproblem from the widget, just in case
        # subppath wasn't a complete subproblem path.
        subpctxt = self.currentSubProblemContext()
        if subpctxt:           # ie, path was complete
            subp = subpctxt.getObject() # CSubProblem object
            mesh = self.currentMesh()
            for fld in allFields:
                fname = fld.name()
                fdef = subp.is_defined_field(fld)
                self.fieldbuttons[(fname, "defined")].set(fdef)
                self.fieldbuttons[(fname, "active")].set_sensitive(fdef)
                self.fieldbuttons[(fname, "inplane")].set_sensitive(fdef)
                if fdef:
                    self.fieldbuttons[(fname, "active")].set(
                        subp.is_active_field(fld))
                    self.fieldbuttons[(fname, "inplane")].set(
                        mesh.in_plane(fld))
                else:                   # field not defined
                    self.fieldbuttons[(fname, "active")].set(0)
                    self.fieldbuttons[(fname, "inplane")].set(0)
        else:                           # no current subproblem
            for button in self.fieldbuttons.values():
                button.set(0)

        meshctxt = self.currentMeshContext()
        if meshctxt:
            self.updateInitializers()
            self.initmsg.hide()
            self.initscroll.show_all()
        else:
            self.initscroll.hide()
            self.initmsg.show()

    # Functions that the TreeView machinery calls to fill in the
    # cells.  These will never be called for undefined fields, so
    # there's no need to check that the mesh or field exists.
    def renderFieldName(self, column, cell_renderer, model, iter):
        debug.mainthreadTest()
        fld = model[iter][0]
        cell_renderer.set_property('text', fld.name())
    def renderFieldInit(self, column, cell_renderer, model, iter):
        debug.mainthreadTest()
        fld = model[iter][0]
        mesh = self.currentMeshContext()
        init = mesh.get_initializer(fld)
        if init:
            cell_renderer.set_property('text', init.shortrepr())
        else:
            cell_renderer.set_property('text', '<uninitialized>')
    def updateInitializers(self):
        debug.mainthreadTest()
        currentField = self.selectedField()
        self.initlist.clear()
        mesh = self.currentMeshContext()
        if mesh:
            for fld in allFields:
                if mesh.is_defined_field(fld):
                    self.initlist.append([fld])
            if currentField is not None:
                for row in range(len(self.initlist)):
                    if self.initlist[row][0] == currentField:
                        self.initview.get_selection().select_path(row)

    def copyfstateCB(self, gtkobj):     # Button callback.
        menuitem = subpmenu.Copy_Field_State
        targetparam = menuitem.get_arg("target")
        
        targetparam.set(self.currentFullSubProblemName())
        if parameterwidgets.getParameters(targetparam,
                                          title="Select a target Subproblem"):
            menuitem.callWithDefaults(source=self.currentFullSubProblemName())

    def copyinitCB(self, gtkobj):       # Button callback
        menuitem = meshmenu.Copy_Field_Initializers
        targetparam = menuitem.get_arg("target")
        targetparam.set(self.currentFullMeshName())
        if parameterwidgets.getParameters(targetparam,
                                          title="Select a target Mesh"):
            menuitem.callWithDefaults(source=self.currentFullMeshName())

    def clearinitCB(self, gtkobj):      # Button callback
        menuitem = meshmenu.Clear_Field_Initializers
        menuitem.callWithDefaults(mesh=self.currentFullMeshName())

    # switchboard ("new who", "Skeleton") or ("new who", "Mesh")
    def newMSorSkeletonOrMesh(self, skeletonname):
        if not self.currentSubProblemContext():
            self.subpwidget.set_value(skeletonname)

##    def newSubProblem(self, subpname):  # switchboard ("new who", "SubProblem")
##        self.set_state(subpname)
##        self.sensitize()
        
##    def subpCB(self, meshpath):         # WhoWidget callback
##        self.set_state(meshpath)
##        self.sensitize()

    def subpwidgetCB(self, interactive):      # switchboard callback
        self.update()
        self.sensitize()

    def reservationChanged(self, whoobj):
        # switchboard "made reservation", "cancelled reservation"
        if  self.currentFullSubProblemName() == whoobj.path():
            self.sensitize()

    def initSelectCB(self, selection):
        self.sensitize()

    def initActivateRowCB(self, treeview, path, col):
        self.initbuttonCB()
            
    # Get the Field currently selected in the initializer TreeView
    def selectedField(self):
        debug.mainthreadTest()
        selection = self.initview.get_selection() # TreeSelection object
        model, iter = selection.get_selected()
        if iter:
            return model[iter][0]

    def initbuttonCB(self, *args):      # gtk button callback
        menuitem = meshmenu.Initialize
        fld = self.selectedField()
        # We can't simply use
        # parameter.getParameters(menuitem.get_arg('initializer'))
        # here, because the menu items parameter accepts members of
        # the FieldInit base class, but we need to create a widget for
        # the appropriate derived class (ScalarFieldInit, et al).
        # It's the derived classes that are the registered classes.
        param = parameter.RegisteredParameter(
            'initializer', fieldinit.fieldInitDict[fld.classname()])

        # Set the parameter value to the current initializer of the
        # appropriate field in the current mesh (context) -- this is
        # the value that the initializer widget will start with.  If
        # it doesn't get set here, it will just have the most
        # recently-set one, which may be for a different mesh.
        param.value = self.currentMeshContext().get_initializer(fld)

        init = parameterwidgets.getParameterValues(
            param,
            title = 'Initialize field ' + self.selectedField().name(),
            scope=self)
        
        if init is not None:
            menuitem(mesh=self.currentFullMeshName(),
                     field=fld,
                     initializer=init[0])

    def initFldCB(self):                # sb "field initialized"
        # This function really should just emit the ListStore signal
        # that causes its TreeView to update.  Presumably that's easy
        # to do, but I don't know how at the moment, so I'll just
        # rebuild the list entirely.
        self.updateInitializers()


#############

fp = FieldPage()
