# -*- python -*-
# $RCSfile: meshPage.py,v $
# $Revision: 1.94 $
# $Author: langer $
# $Date: 2007/02/06 22:47:44 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from oof2.SWIG.common import switchboard
from oof2.SWIG.engine import masterelement
from oof2.common import debug
from oof2.common import guitop
from oof2.common import labeltree
from oof2.common import microstructure
from oof2.common.IO import mainmenu
from oof2.common.IO import parameter
from oof2.common.IO import reporter
from oof2.common.IO.GUI import chooser
from oof2.common.IO.GUI import gtklogger
from oof2.common.IO.GUI import gtkutils
from oof2.common.IO.GUI import historian
from oof2.common.IO.GUI import oofGUI
from oof2.common.IO.GUI import parameterwidgets
from oof2.common.IO.GUI import regclassfactory
from oof2.common.IO.GUI import whowidget
from oof2.engine import meshmod
from oof2.engine import skeletoncontext
import gtk
import oof2.engine.mesh
import string

meshmenu = mainmenu.OOF.Mesh

class MeshPage(oofGUI.MainPage):
    def __init__(self):
        self.built = False
        oofGUI.MainPage.__init__(self, name="FE Mesh", ordering=200,
                                 tip="Create a Finite Element Mesh from a Skeleton.")
        mainbox = gtk.VBox(spacing=2)
        self.gtk.add(mainbox)

        align = gtk.Alignment(xalign=0.5)
        mainbox.pack_start(align, expand=0, fill=0)
        centerbox = gtk.HBox(spacing=3)
        align.add(centerbox)
        self.meshwidget = whowidget.WhoWidget(oof2.engine.mesh.meshes,
                                              callback=self.meshCB,
                                              scope=self)
        label = gtk.Label("Microstructure=")
        label.set_alignment(1.0, 0.5)
        centerbox.pack_start(label, expand=0, fill=0)
        centerbox.pack_start(self.meshwidget.gtk[0], expand=0, fill=0)

        label = gtk.Label("Skeleton=")
        label.set_alignment(1.0, 0.5)
        centerbox.pack_start(label, expand=0, fill=0)
        centerbox.pack_start(self.meshwidget.gtk[1], expand=0, fill=0)

        label = gtk.Label("Mesh=")
        label.set_alignment(1.0, 0.5)
        centerbox.pack_start(label, expand=0, fill=0)
        centerbox.pack_start(self.meshwidget.gtk[2], expand=0, fill=0)

        # Centered box of buttons
        align = gtk.Alignment(xalign=0.5)
        mainbox.pack_start(align, expand=0, fill=0)
        bbox = gtk.HBox(homogeneous=1, spacing=3)
        align.add(bbox)
        self.newbutton = gtkutils.stockButton(gtk.STOCK_NEW, "New...")
        gtklogger.setWidgetName(self.newbutton, 'New')
        gtklogger.connect(self.newbutton, 'clicked', self.newCB)
        guitop.top().tooltip(self.newbutton, "Create a new mesh from the current skeleton.")
        bbox.pack_start(self.newbutton, expand=0, fill=1)
        
        self.renamebutton = gtkutils.stockButton(gtk.STOCK_EDIT, "Rename...")
        gtklogger.setWidgetName(self.renamebutton, 'Rename')
        gtklogger.connect(self.renamebutton, 'clicked', self.renameCB)
        guitop.top().tooltip(self.renamebutton, "Rename the current mesh.")   
        bbox.pack_start(self.renamebutton, expand=0, fill=1)
        
        self.copybutton = gtkutils.stockButton(gtk.STOCK_COPY, "Copy...")
        gtklogger.setWidgetName(self.copybutton, 'Copy')
        gtklogger.connect(self.copybutton, 'clicked', self.copyCB)
        guitop.top().tooltip(self.copybutton, "Copy the current mesh.")
        bbox.pack_start(self.copybutton, expand=0, fill=1)
        
        self.deletebutton = gtkutils.stockButton(gtk.STOCK_DELETE, "Delete")
        gtklogger.setWidgetName(self.deletebutton, 'Delete')
        gtklogger.connect(self.deletebutton, 'clicked', self.deleteCB)
        guitop.top().tooltip(self.deletebutton, "Delete the current mesh.")
        bbox.pack_start(self.deletebutton, expand=0, fill=1)
        
        self.savebutton = gtkutils.stockButton(gtk.STOCK_SAVE, "Save...")
        gtklogger.setWidgetName(self.savebutton, 'Save')
        gtklogger.connect(self.savebutton, 'clicked', self.saveCB)
        guitop.top().tooltip(self.savebutton,
                             "Save the current mesh to a file.")
        bbox.pack_start(self.savebutton, expand=0, fill=1)

        mainpane = gtk.HPaned()
        gtklogger.setWidgetName(mainpane, 'Pane')
        mainbox.pack_start(mainpane, expand=1, fill=1)
##        leftbox = gtk.VBox()
        leftbox = gtk.VPaned()
        mainpane.pack1(leftbox, resize=1, shrink=0)
        
        infoframe = gtk.Frame('Mesh Information')
        infoframe.set_shadow_type(gtk.SHADOW_IN)
##        leftbox.pack_start(infoframe, expand=0, fill=1)
        leftbox.pack1(infoframe, resize=1, shrink=1)
        gtklogger.connect_passive(infoframe, 'size-allocate')
        scroll = gtk.ScrolledWindow()
        gtklogger.logScrollBars(scroll, "MeshInfo")
        scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        scroll.set_shadow_type(gtk.SHADOW_IN)
        infoframe.add(scroll)
        self.infoarea = gtk.TextView()
        self.infoarea.set_cursor_visible(False)
        self.infoarea.set_editable(False)
        scroll.add(self.infoarea)

        # Subproblem creation, deletion, etc.
        subprobframe = gtk.Frame('Subproblems')
        gtklogger.setWidgetName(subprobframe, 'Subproblems')
        subprobframe.set_shadow_type(gtk.SHADOW_IN)
##        leftbox.pack_start(subprobframe, expand=1, fill=1)
        leftbox.pack2(subprobframe, resize=1, shrink=1)
        subpbox = gtk.VBox()
        subprobframe.add(subpbox)
        self.subpchooser = chooser.ScrolledChooserListWidget(
            callback=self.subpchooserCB,
            dbcallback=self.subprobEditCB,
            name="subprobChooser")
        subpbox.pack_start(self.subpchooser.gtk, expand=1, fill=1)

        subpbuttons1 = gtk.HBox(homogeneous=True, spacing=2)
        subpbuttons2 = gtk.HBox(homogeneous=True, spacing=2)
        subpbox.pack_start(subpbuttons1, expand=0, fill=0)
        subpbox.pack_start(subpbuttons2, expand=0, fill=0)

        self.subprobNew = gtkutils.stockButton(gtk.STOCK_NEW, "New...")
        gtklogger.setWidgetName(self.subprobNew, "New")
        gtklogger.connect(self.subprobNew, "clicked", self.subprobNewCB)
        guitop.top().tooltip(self.subprobNew, "Create a new subproblem.")
        subpbuttons1.pack_start(self.subprobNew, expand=1, fill=1)

        self.subprobRename = gtk.Button("Rename...")
        gtklogger.setWidgetName(self.subprobRename, "Rename")
        gtklogger.connect(self.subprobRename, "clicked", self.subprobRenameCB)
        guitop.top().tooltip(self.subprobRename,
                             "Rename the selected subproblem")
        subpbuttons1.pack_start(self.subprobRename, expand=1, fill=1)

        self.subprobEdit = gtkutils.stockButton(gtk.STOCK_EDIT, "Edit...")
        gtklogger.setWidgetName(self.subprobEdit, "Edit")
        gtklogger.connect(self.subprobEdit, 'clicked', self.subprobEditCB)
        guitop.top().tooltip(self.subprobEdit, "Edit the selected subproblem.")
        subpbuttons1.pack_start(self.subprobEdit, expand=1, fill=1)

        self.subprobCopy = gtkutils.stockButton(gtk.STOCK_COPY, "Copy...")
        gtklogger.setWidgetName(self.subprobCopy, "Copy")
        gtklogger.connect(self.subprobCopy, "clicked", self.subprobCopyCB)
        guitop.top().tooltip(self.subprobCopy, "Copy the selected subproblem.")
        subpbuttons2.pack_start(self.subprobCopy, expand=1, fill=1)

##        subpbuttons2.pack_start(gtk.HBox(), expand=1, fill=1) # filler
        self.subprobInfo = gtk.Button("Info")
        gtklogger.setWidgetName(self.subprobInfo, "Info")
        gtklogger.connect(self.subprobInfo, 'clicked', self.subprobInfoCB)
        guitop.top().tooltip(self.subprobInfo,
                             "Print information about the selected subproblem")
        subpbuttons2.pack_start(self.subprobInfo, expand=1, fill=1)
        
        self.subprobDelete = gtkutils.stockButton(gtk.STOCK_DELETE, "Delete")
        gtklogger.setWidgetName(self.subprobDelete, "Delete")
        gtklogger.connect(self.subprobDelete, "clicked", self.subprobDeleteCB)
        guitop.top().tooltip(self.subprobDelete,
                             "Delete the selected subproblem.")
        subpbuttons2.pack_start(self.subprobDelete, expand=1, fill=1)
        
        # Right hand side for element operations
        
        elementopsframe = gtk.Frame(label="Mesh Operations")
        gtklogger.setWidgetName(elementopsframe, 'ElementOps')
        elementopsframe.set_shadow_type(gtk.SHADOW_IN)
        mainpane.pack2(elementopsframe, resize=0, shrink=0)
        elementopsbox = gtk.VBox(spacing=3)
        elementopsframe.add(elementopsbox)
        self.elementops = regclassfactory.RegisteredClassFactory(
            meshmod.MeshModification.registry,
            title="Method:",
            callback=self.elementopsCB,
            expand=0, fill=0, scope=self, name="Method")
        elementopsbox.pack_start(self.elementops.gtk, expand=1, fill=1)

        self.historian = historian.Historian(self.elementops.set,
                                             self.sensitizeHistory,
                                             setCBkwargs={'interactive':1})
        # Prev, OK, Next
        hbox = gtk.HBox()
        elementopsbox.pack_start(hbox, expand=0, fill=0, padding=2)
        self.prevbutton = gtkutils.prevButton()
        gtklogger.connect(self.prevbutton, 'clicked', self.prevCB)
        guitop.top().tooltip(self.prevbutton,
                             "Recall the previous mesh element operation.")
        hbox.pack_start(self.prevbutton, expand=0, fill=0, padding=2)

        self.okbutton = gtk.Button(stock=gtk.STOCK_OK)
        gtklogger.setWidgetName(self.okbutton, 'OK')
        gtklogger.connect(self.okbutton, 'clicked', self.okCB)
        guitop.top().tooltip(self.okbutton,
                          'Perform the mesh element operation defined above.')
        hbox.pack_start(self.okbutton, expand=1, fill=1, padding=5)

        self.nextbutton = gtkutils.nextButton()
        gtklogger.connect(self.nextbutton, 'clicked', self.nextCB)
        guitop.top().tooltip(self.nextbutton,
                             'Recall the next mesh element operation.')
        hbox.pack_start(self.nextbutton, expand=0, fill=0, padding=2)

        self.built = True

        switchboard.requestCallbackMain("Mesh modified",
                                        self.recordModifier)
        switchboard.requestCallbackMain(("new who", "Microstructure"),
                                        self.newMSorSkeleton)
        switchboard.requestCallbackMain(("new who", "Skeleton"),
                                        self.newMSorSkeleton)
        switchboard.requestCallbackMain(("new who", "Mesh"),
                                        self.newMesh)
        switchboard.requestCallbackMain(("new who", "SubProblem"),
                                        self.newSubProblem)
        switchboard.requestCallbackMain(("rename who", "SubProblem"),
                                        self.renamedSubProblem)
        switchboard.requestCallbackMain(("remove who", "SubProblem"),
                                         self.removeSubProblem)
        switchboard.requestCallbackMain(self.meshwidget, self.meshwidgetCB)
        switchboard.requestCallbackMain("equation activated",
                                        self.equationCB)
        switchboard.requestCallbackMain("mesh boundaries changed",
                                        self.newMeshBoundaries)

        switchboard.requestCallbackMain(('validity', self.elementops),
                                        self.validityChangeCB)
        
        self.sensitize()
        self.sensitizeHistory()

    def show(self):
        debug.mainthreadTest()
        self.gtk.show_all()
        self.update()

    def newMeshBoundaries(self, mesh):
        if mesh==self.currentMesh():
            self.update()
            self.sensitize()
            
    #######################

    def currentSkeletonFullName(self):
        return self.meshwidget.get_value(depth=2)
    def currentSkeletonContext(self):
        try:
            return skeletoncontext.skeletonContexts[
                self.currentSkeletonFullName()]
        except KeyError:
            return None

    def currentFullMeshName(self):
        return self.meshwidget.get_value()
    def currentMeshName(self):
        path = labeltree.makePath(self.currentFullMeshName())
        if path:
            return path[2]
    def currentMeshContext(self):
        try:
            return oof2.engine.mesh.meshes[self.currentFullMeshName()]
        except KeyError:
            return None
    def currentMesh(self):
        ctxt = self.currentMeshContext()
        if ctxt:
            return ctxt.getObject()

    def currentSubProblemName(self):
        return self.subpchooser.get_value()
    def currentSubProblemContext(self):
        meshctxt = self.currentMeshContext()
        if meshctxt is not None:
            try:
                return meshctxt.get_subproblem(self.currentSubProblemName())
            except KeyError:
                return None
    def currentFullSubProblemName(self):
        ctxt = self.currentSubProblemContext()
        if ctxt:
            return ctxt.path()
            
    def sensitize(self):
        debug.mainthreadTest()
        skelok = self.currentSkeletonContext() is not None
        meshok = self.currentMeshContext() is not None
        self.newbutton.set_sensitive(skelok)
        self.deletebutton.set_sensitive(meshok)
        self.renamebutton.set_sensitive(meshok)
        self.copybutton.set_sensitive(meshok)
        self.savebutton.set_sensitive(meshok)
        self.okbutton.set_sensitive(meshok and self.elementops.isValid())
        gtklogger.checkpoint("mesh page sensitized")
        self.sensitizeSubProblems()

    def sensitizeHistory(self):
        debug.mainthreadTest()
        self.nextbutton.set_sensitive(self.historian.nextSensitive())
        self.prevbutton.set_sensitive(self.historian.prevSensitive())
        
    def update(self):
        self.set_state(self.currentFullMeshName())

    def recordModifier(self, path, modifier):  # callback for "Mesh modified"
        if modifier:
            self.historian.record(modifier)
        # might as well do the update for Mesh Information pane.
        self.set_state(path)
        
    def set_state(self, meshpath):  # widget update & information update
        debug.mainthreadTest()
        path = labeltree.makePath(meshpath)
        self.meshwidget.set_value(path)
        buffer = self.infoarea.get_buffer()
        buffer.delete(buffer.get_start_iter(), buffer.get_end_iter())
        if path and path[-1]:           # ie, mesh name is not empty
            themesh = oof2.engine.mesh.meshes[path]
            skel = themesh.getSkeleton()
            buffer.insert(buffer.get_end_iter(),
                          "No. of Nodes:\t%d\n" % themesh.nnodes())
            buffer.insert(buffer.get_end_iter(),
                          "No. of Elements:\t%d\n" % themesh.nelements())
            masterelementenums = masterelement.getMasterElementEnums()
            sides = masterelementenums.keys() # list of element geometries
            sides.sort()
            counts = [0]*(max(sides)+1)
            for elem in skel.element_iterator():
                counts[elem.nnodes()] += 1
            for nsides in sides:
                buffer.insert(buffer.get_end_iter(),
                              "%d sided element:\t%s (%d)\n"
                              % (nsides, themesh.getMasterElementType(nsides),
                                 counts[nsides]))
        else:                           # no current mesh
            buffer.insert(buffer.get_end_iter(), "No mesh!")
        self.set_subproblem_state()
        self.sensitize()
        self.sensitizeHistory()

    def newMSorSkeleton(self, path):
        # switchboard ("new who", "Microstructure") or ("new who", "Skeleton")
        if not self.currentMesh():
            self.meshwidget.set_value(path)

    def newMesh(self, meshname):        # switchboard ("new who", "Mesh")
        self.set_state(meshname)
        self.sensitize()
        
    def meshCB(self, meshpath):         # WhoWidget callback
        self.set_state(meshpath)
        self.sensitize()
    def meshwidgetCB(self, interactive): # switchboard widget callback
        self.update()
        self.sensitize()

    def equationCB(self, *args):  # switchboard "equation activated"
        switchboard.notify(self.meshwidget, interactive=1)

    def newCB(self, *args):             # gtk button callback
        menuitem = mainmenu.OOF.Mesh.New
        params = filter(lambda x: x.name !='skeleton', menuitem.params)
        if parameterwidgets.getParameters(title='Create a new mesh',
                                          scope=self, *params):
            menuitem.callWithDefaults(skeleton=self.currentSkeletonFullName())

    def deleteCB(self, *args):          # gtk button callback
        if reporter.query(
            "Really delete %s?"%self.currentFullMeshName(),
            "No", default="Yes") == "Yes":
            meshmenu.Delete(mesh=self.currentFullMeshName())

##    def removeMesh(self, path):    # switchboard ("remove who", "Mesh")
##        self.update()
##        self.sensitize()

    def copyCB(self, *args):            # gtk button callback
        menuitem = mainmenu.OOF.Mesh.Copy
        nameparam = menuitem.get_arg("name")
        fieldparam = menuitem.get_arg("copy_field")
        eqnparam = menuitem.get_arg("copy_equation")
        bcparam = menuitem.get_arg("copy_bc")
        if parameterwidgets.getParameters(nameparam, fieldparam, eqnparam,
                                          bcparam, title='Copy a mesh'):
            menuitem.callWithDefaults(mesh=self.currentFullMeshName())
        
    def renameCB(self, *args):          # gtk button callback
        menuitem = meshmenu.Rename
        namearg = menuitem.get_arg('name')
        curmeshpath = self.currentFullMeshName()
        namearg.value = labeltree.makePath(curmeshpath)[-1]
        if parameterwidgets.getParameters(namearg,
                                          title='Rename mesh '+namearg.value):
            menuitem.callWithDefaults(mesh=curmeshpath)

    def saveCB(self, *args):
        menuitem = mainmenu.OOF.File.Save.Mesh
        meshname = self.meshwidget.get_value()
        file = fileselector.fileSelector(
            ident='SaveMeshFromPage',
            mode='w',
            title='Save Mesh "%s"' % meshname,
            params=[menuitem.get_arg('format')])
        if file is not None:
            mode = fileselector.get_file_mode(file)
            if mode is not None:
                menuitem.callWithDefaults(filename=file, mode=mode,
                                          mesh=meshname)
    def prevCB(self, gtkobj):
        self.historian.prevCB()

    def nextCB(self, gtkobj):
        self.historian.nextCB()

    def elementopsCB(self, reg):
        self.historian.stateChangeCB(reg)
        self.sensitize()

    def okCB(self, gtkobj):
        path = self.meshwidget.get_value()
        modifier = self.elementops.get_value()
        if path and modifier:
            mainmenu.OOF.Mesh.Modify(mesh=path, modifier=modifier)

    def validityChangeCB(self, validity):
        self.sensitize()

    ######################

    # subproblem callbacks and sensitization

    def set_subproblem_state(self, subprobname=None):
        meshctxt = self.currentMeshContext()
        if meshctxt is not None:
            self.subpchooser.update([s.name() for s in meshctxt.subproblems()])
            if subprobname is not None:
                self.subpchooser.set_selection(subprobname)
        else:
            self.subpchooser.update([])

    def subprobNewCB(self, gtkobj):
        menuitem = mainmenu.OOF.Subproblem.New
        params = filter(lambda x: x.name != 'mesh', menuitem.params)
        if parameterwidgets.getParameters(title='Create a new subproblem',
                                          scope=self, *params):
            menuitem.callWithDefaults(mesh=self.currentFullMeshName())
    def subprobCopyCB(self, gtkobj):
        menuitem = mainmenu.OOF.Subproblem.Copy
        # Initialize the 'mesh' parameter to the current mesh, but
        # allow the user to change it.  Usually the subproblem will be
        # copied to the current mesh, but not always.
        meshparam = menuitem.get_arg('mesh')
        meshparam.value = self.currentFullMeshName()
        params = filter(lambda x: x.name != 'subproblem', menuitem.params)
        if parameterwidgets.getParameters(title='Copy a subproblem',
                                          scope=self, *params):
            menuitem.callWithDefaults(
                subproblem=self.currentFullSubProblemName())
    def subprobRenameCB(self, gtkobj):
        menuitem = mainmenu.OOF.Subproblem.Rename
        namearg = menuitem.get_arg('name')
        cursubprob = self.currentFullSubProblemName()
        debug.fmsg("cursubprob=", self.currentFullSubProblemName())
        namearg.value = labeltree.makePath(cursubprob)[-1]
        if parameterwidgets.getParameters(
            namearg, title="Rename subproblem " + namearg.value):
            menuitem.callWithDefaults(subproblem=cursubprob)

    def subprobInfoCB(self, gtkobj):
        mainmenu.OOF.Subproblem.Info(
            subproblem=self.currentFullSubProblemName())

    def subprobDeleteCB(self, gtkobj):
        if reporter.query(
            "Really delete %s" % self.currentFullSubProblemName(),
            "No", default="Yes") == "Yes":
            mainmenu.OOF.Subproblem.Delete(
                subproblem=self.currentFullSubProblemName())

    def subprobEditCB(self, gtkobj):
        subproblemname = self.currentFullSubProblemName()
        if subproblemname:
            menuitem = mainmenu.OOF.Subproblem.Edit
            subpctxt = oof2.engine.subproblemcontext.subproblems[subproblemname]
            subpobj = subpctxt.getObject()
            subpparam = menuitem.get_arg('subproblem')
            subpparam.set(subpobj)
            if parameterwidgets.getParameters(
                subpparam, title="Edit Subproblem definition", scope=self):
                menuitem.callWithDefaults(name=subproblemname)
        

    def subpchooserCB(self, subp, interactive):
        if self.built:
            self.sensitizeSubProblems()

    def sensitizeSubProblems(self):
        debug.mainthreadTest()
        subpok = self.subpchooser.get_value() is not None
        defaultsubp = (self.subpchooser.get_value() ==
                       oof2.engine.mesh.defaultSubProblemName)
        meshok = self.currentMesh() is not None
        self.subprobNew.set_sensitive(meshok)
        self.subprobRename.set_sensitive(subpok and not defaultsubp)
        self.subprobCopy.set_sensitive(subpok)
        self.subprobDelete.set_sensitive(subpok and not defaultsubp)
        self.subprobEdit.set_sensitive(subpok and not defaultsubp)
        self.subprobInfo.set_sensitive(subpok)
        gtklogger.checkpoint("mesh page subproblems sensitized")

    def newSubProblem(self, subproblempath): # sb ("new who", "SubProblem")
        path = labeltree.makePath(subproblempath)
        if labeltree.makePath(self.currentFullMeshName()) == path[:-1]:
            self.set_subproblem_state(path[-1])
            self.sensitizeSubProblems()
    def renamedSubProblem(self, oldpath, newname):
        # switchboard ("rename who", "SubProblem")
        path = labeltree.makePath(oldpath)
        if labeltree.makePath(self.currentFullMeshName()) == path[:-1]:
            self.set_subproblem_state(newname)
    def removeSubProblem(self, path):
        if labeltree.makePath(self.currentFullMeshName()) == path[:-1]:
            self.set_subproblem_state()
        

#############
        
mp = MeshPage()

###########################################################

from oof2.common.IO.GUI import fileselector

def _save_mesh(menuitem):
    if len(oof2.engine.mesh.meshes) > 0:
        file = fileselector.fileSelector(
            ident='SaveMeshFromMenu',
            title='Save Mesh',
            mode='w',
            params=[menuitem.get_arg('mesh'), menuitem.get_arg('format')]
            )
        if not file is None:
            mode = fileselector.get_file_mode(file)
            if mode is not None:
                menuitem.callWithDefaults(filename=file, mode=mode)
    else:
        reporter.report("No mesh?")

mainmenu.OOF.File.Save.Mesh.add_gui_callback(_save_mesh)
