# -*- python -*-
# $RCSfile: solverPage.py,v $
# $Revision: 1.84 $
# $Author: langer $
# $Date: 2007/01/08 18:27:31 $


# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 


from oof2.SWIG.common import config
from oof2.SWIG.common import switchboard
from oof2.SWIG.engine import preconditioner
from oof2.SWIG.engine import solver
from oof2.SWIG.engine import solverdriver
from oof2.common import debug
from oof2.common import guitop
from oof2.common import labeltree
from oof2.common import microstructure
from oof2.common import utils
from oof2.common.IO import mainmenu
from oof2.common.IO import parameter, reporter
from oof2.common.IO import whoville
from oof2.common.IO.GUI import chooser
from oof2.common.IO.GUI import gtklogger
from oof2.common.IO.GUI import gtkutils
from oof2.common.IO.GUI import oofGUI
from oof2.common.IO.GUI import regclassfactory
from oof2.common.IO.GUI import whowidget
from oof2.engine import mesh
from oof2.engine import skeletoncontext
from oof2.engine import subproblemcontext
from oof2.engine import symstate
from oof2.engine import timedrivers
from oof2.engine.IO import solvermenu
import gtk
import string

OOF = mainmenu.OOF

class SolverSelectFactory(regclassfactory.RegisteredClassFactory):
    def __init__(self, registry, obj=None, title=None,
                 callback=None, fill=0, expand=0, scope=None, name=None,
                 widgetdict={}, *args, **kwargs):
        self.subpwidget = scope.findWidget(
            lambda w: isinstance(w, whowidget.WhoWidget)
            and w.whoclass is subproblemcontext.subproblems)
        self.sbcb = switchboard.requestCallbackMain(self.subpwidget,
                                                    self.mwCB)
        self.subpobj = None

        regclassfactory.RegisteredClassFactory.__init__(
            self, registry, obj=obj, title=title, callback=callback,
            fill=fill, expand=expand, scope=scope, name=name,
            widgetdict=widgetdict,
            *args, **kwargs)

    def cleanUp(self):
        switchboard.removeCallback(self.sbcb)
        regclassfactory.RegisteredClassFactory.cleanUp(self)

    def mwCB(self, *args, **kwargs):
        self.subpobj = None
        if self.subpwidget.isValid():
            self.subpobj = subproblemcontext.subproblems[
                self.subpwidget.get_value()]

    def refresh(self):
        self.update(self.registry)
            
    def includeRegistration(self, reg):
        # If the matrix is not symmetric, then exclude the solvers
        # which *are* symmetric, otherwise, just include all
        # non-secret solvers.
        if self.subpobj:
            if self.subpobj.get_symmetry_state() != symstate.SYMMETRIC:
                return (not reg.symmetric) and (not reg.secret)
            else:
                return not reg.secret
        else:
            return not reg.secret
            
def _makeLinearSolverWidget(self, scope=None):
    return SolverSelectFactory(self.registry, scope=scope, name=self.name)

solver.LinearSolverParameter.makeWidget = _makeLinearSolverWidget
            

class SolverPage(oofGUI.MainPage):
    def __init__(self):
        self.sbcallbacks = [
            switchboard.requestCallbackMain("subproblem changed",
                                            self.subproblemChanged),
            switchboard.requestCallbackMain("mesh changed", self.meshChanged),
            switchboard.requestCallbackMain("mesh solved",
                                            self.meshSolved),
            switchboard.requestCallbackMain("made reservation",
                                            self.reservationChanged),
            switchboard.requestCallbackMain("cancelled reservation",
                                            self.reservationChanged),
            switchboard.requestCallbackMain(("new who", "Skeleton"),
                                            self.newSkeleton),
##            switchboard.requestCallbackMain(("new who", "Mesh"), self.newMesh),
##            switchboard.requestCallbackMain(("new who", "SubProblem"),
##                                            self.newSubProblem)
##            switchboard.requestCallbackMain(("remove who", "SubProblem"),
##                                            self.removeSubProblem)
            ]
            
        oofGUI.MainPage.__init__(self, name="Solver",
                                 ordering=240,
                                 tip="Set mesh and time driver to find the solution of active fields and partial differential equations")
        mainbox = gtk.VBox()
        self.gtk.add(mainbox)

        align = gtk.Alignment(xalign=0.5)
        mainbox.pack_start(align, expand=0, fill=0)
        centerbox = gtk.HBox(spacing=3)
        align.add(centerbox)
        self.subpwidget = whowidget.WhoWidget(whoville.getClass('SubProblem'),
                                              ##callback=self.meshCB,
                                              scope=self)
        switchboard.requestCallbackMain(self.subpwidget, self.subpwidgetCB)
        label = gtk.Label("Microstructure=")
        label.set_alignment(1.0, 0.5)
        centerbox.pack_start(label, expand=0, fill=0)
        centerbox.pack_start(self.subpwidget.gtk[0], expand=0, fill=0)
        label = gtk.Label("Skeleton=")
        label.set_alignment(1.0, 0.5)
        centerbox.pack_start(label, expand=0, fill=0)
        centerbox.pack_start(self.subpwidget.gtk[1], expand=0, fill=0)
        label = gtk.Label("Mesh=")
        label.set_alignment(1.0, 0.5)
        centerbox.pack_start(label, expand=0, fill=0)
        centerbox.pack_start(self.subpwidget.gtk[2], expand=0, fill=0)
        label = gtk.Label("SubProblem=")
        label.set_alignment(1.0, 0.5)
        centerbox.pack_start(label, expand=0, fill=0)
        centerbox.pack_start(self.subpwidget.gtk[3], expand=0, fill=0)
        
        panes = gtk.HPaned()
        mainbox.pack_start(panes, expand=1, fill=1)

        statusframe = gtk.Frame("Mesh/Subproblem Status")
        statusframe.set_shadow_type(gtk.SHADOW_IN)
        panes.pack1(statusframe, resize=True)
        scroll = gtk.ScrolledWindow()
        gtklogger.logScrollBars(scroll, "StatusScroll")
        scroll.set_shadow_type(gtk.SHADOW_IN)
        scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        statusframe.add(scroll)
        self.statustext = gtk.TextView()
        gtklogger.setWidgetName(self.statustext, 'Status')
        self.statustext.set_wrap_mode(gtk.WRAP_WORD)
        self.statustext.set_editable(False)
        self.statustext.set_cursor_visible(False)
        scroll.add(self.statustext)

        ## solver component selection
        solverVwindow = gtk.VBox()
        panes.pack2(solverVwindow, resize=0, shrink=0)
        ## solver driver selection
        if config.devel()>=1:
            solverdriverframe = gtk.Frame("MatrixDriver")
        else:
            solverdriverframe = gtk.Frame("Solver")
        solverdriverframe.set_shadow_type(gtk.SHADOW_IN)
        ## panes.pack2(solverdriverframe, shrink=0)
        solverVwindow.pack_start(solverdriverframe, expand=1, fill=1)
        self.driverSelectorFactory = regclassfactory.RegisteredClassFactory(
            solverdriver.Driver.registry, callback=self.newDriver, scope=self,
            name="Driver")
        solverdriverframe.add(self.driverSelectorFactory.gtk)

        self.newDriver()
        
        if config.devel()>=1:
            ## time_driver selection
            self.time_driverSelectorFactory = regclassfactory.RegisteredClassFactory(
                timedrivers.TimeDriver.registry,
                name="TimeDriver")
            time_driverframe = gtk.Frame("TimeDriver")
            time_driverframe.set_shadow_type(gtk.SHADOW_IN)
            solverVwindow.pack_start(time_driverframe, expand=1, fill=1)
            time_driverframe.add(self.time_driverSelectorFactory.gtk)
        
        solverbuttons = gtk.HBox(spacing=2, homogeneous=1)
        solverVwindow.pack_start(solverbuttons, expand=0, fill=0)
        if config.devel()>=1:
            self.resetButton = gtk.Button("Reset")
            gtklogger.setWidgetName(self.resetButton, 'Reset')
            gtklogger.connect(self.resetButton, "clicked", self.resetCB)
            guitop.top().tooltip(self.resetButton,
                        "set fields back to their (unsolved) initial state")
            solverbuttons.pack_start(self.resetButton, expand=1, fill=1)

        self.solveButton = gtkutils.stockButton(gtk.STOCK_EXECUTE, "Solve")
        gtklogger.setWidgetName(self.solveButton, 'Solve')
        gtklogger.connect(self.solveButton, "clicked", self.solveCB)
        guitop.top().tooltip(self.solveButton,
                             "Apply solver to the current subproblem")
        solverbuttons.pack_start(self.solveButton, expand=1, fill=1)
        
        self.sensitize()
    def show(self):
        debug.mainthreadTest()
        self.gtk.show_all()
        self.update()

    #######################

    # When the driver widget switches over, the solver widget is
    # reconstructed -- get the new one.
    def newDriver(self, *args):
        self.solverwidget = self.driverSelectorFactory.findWidget(
            lambda w: isinstance(w, SolverSelectFactory) )
        
    def currentMSName(self):
        path = labeltree.makePath(self.subpwidget.get_value(depth=1))
        if path:
            return path[0]
    def currentMS(self):
        msname = self.currentMSName()
        if msname:
            return microstructure.microStructures[msname].getObject()

    def currentSkeletonName(self):
        path = labeltree.makePath(self.subpwidget.get_value(depth=2))
        if path:
            return path[1]
    def currentSkeleton(self):
        ms = self.currentMS()
        skelname = self.currentSkeletonName()
        if skelname:
            return skeletoncontext.getSkeleton(ms, skelname)

    def currentFullMeshName(self):
        pname = self.subpwidget.get_value(depth=3)
        path = labeltree.makePath(pname)
        if path and len(path) >= 3:
            return path[0] + ':' + path[1] + ':' + path[2]
    def currentMeshName(self):
        path = labeltree.makePath(self.subpwidget.get_value(depth=3))
        if path:
            return path[2]
    def currentMeshContext(self):
        try:
            return mesh.meshes[self.currentFullMeshName()]
        except KeyError:
            return None
    def currentMesh(self):
        meshcontext = self.currentMeshContext()
        if meshcontext is not None:
            return meshcontext.getObject()

    def currentFullSubProblemName(self):
        return self.subpwidget.get_value()
    def currentSubProblemContext(self):
        try:
            return subproblemcontext.subproblems[
                self.currentFullSubProblemName()]
        except KeyError:
            return None
    def currentSubProblem(self):
        subpcontext = self.currentSubProblemContext()
        if subpcontext is not None:
            return subpcontext.getObject()

    def sensitize(self):
        debug.mainthreadTest()
        subp = self.currentSubProblem()
        subpcontext = self.currentSubProblemContext()
        subpok = subp is not None
        self.solveButton.set_sensitive(
            subpok and                  # we have a subproblem
            not subpcontext.query_reservation() and # ... and it's not busy
            subpcontext.materialsConsistent() and # ... and it's computable
            subp.n_active_eqns() > 0 and
            subp.n_active_fields() > 0
            )
        if config.devel()>=1:
            self.resetButton.set_sensitive(
                subpok and                  # we have a subproblem
                not subpcontext.query_reservation() # ... and it's not busy
                )
            
    def update(self):
        self.set_state(self.currentFullSubProblemName())
        self.updateStatus()

    def updateStatus(self):
        debug.mainthreadTest()
        subpctxt = self.currentSubProblemContext()
        if subpctxt is not None:
            self.statustext.get_buffer().set_text(subpctxt.status())
        else:
            self.statustext.get_buffer().set_text("No mesh!")
        self.solverwidget.refresh()
        
    def set_state(self, meshpath):
        path = labeltree.makePath(meshpath)
        self.subpwidget.set_value(path)
        self.updateStatus()
        

    def newSkeleton(self, skeletonname): # switchboard ("new who", "Skeleton")
        if not self.currentSubProblem():
            self.subpwidget.set_value(skeletonname)

##    def newMesh(self, meshname):        # sb ("new who", "Mesh")
##        if not self.currentSubProblemContext():
##            self.subpwidget.set_value(meshname)

##    def newSubProblem(self, subpname):  # sb ("new who", "SubProblem")
##        self.set_state(subpname)
##        self.sensitize()

##    def meshCB(self, meshpath):         # WhoWidget callback
##        self.set_state(meshpath)
##        self.sensitize()

    def subpwidgetCB(self, interactive):
        self.update()
        self.sensitize()

##    def removeSubProblem(self, path):    # sb ("remove who", "SubProblem")
##        self.update()
##        self.sensitize()

    def reservationChanged(self, whoobj):
        # switchboard "made reservation", "cancelled reservation"
        if  self.currentFullSubProblemName() == whoobj.path():
            self.sensitize()

    def subproblemChanged(self, subpctxt): # switchboard "subproblem changed"
        if self.currentSubProblemContext() is subpctxt:
            self.sensitize()
            self.updateStatus()

    def meshChanged(self, meshctxt):    # switchboard "mesh changed"
        if self.currentMeshContext() is meshctxt:
            self.sensitize()
            self.updateStatus()

    def meshSolved(self, subpctxt):
        if self.currentSubProblemContext() is subpctxt:
            self.sensitize()
            self.updateStatus()

    ###############################

    def solveCB(self, gtkobj):    # "Solve" button callback.
        subproblem_selection = self.currentFullSubProblemName()
        solver_selection = self.driverSelectorFactory.get_value()
        if config.devel()>=1:
            time_driver_selection = self.time_driverSelectorFactory.get_value()
            solvermenu.solvermenu.Solve(subproblem = subproblem_selection,
                                        matrixdriver=solver_selection,
                                        timedriver=time_driver_selection)
        else:
            solvermenu.solvermenu.Solve(subproblem = subproblem_selection,
                                        solver=solver_selection)
            
    def resetCB(self, gtkobj):
        subproblem_selection = self.currentFullSubProblemName()
        solvermenu.solvermenu.Reset(subproblem = subproblem_selection)
        
#####################################################################

# ***************************************************************
# PETScII
# ***************************************************************
if config.petsc():
    from oof2.SWIG.engine.PETSc import petsc_solverdriver
    from oof2.SWIG.engine.PETSc import petsc_solver
    petsc_solver.PETScLinearSolverParameter.makeWidget = _makeLinearSolverWidget
# ***************************************************************

solverpage = SolverPage()



