# -*- python -*-
# $RCSfile: elementselectdisplay.py,v $
# $Revision: 1.16 $
# $Author: langer $
# $Date: 2005/08/08 21:39:33 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from oof2.common import color
from oof2.common import debug
from oof2.common import primitives
from oof2.common import registeredclass
from oof2.common.IO import display
from oof2.common.IO import parameter
from oof2.common.IO import xmlmenudump

class SkeletonElementSelectionDisplay(display.DisplayMethod):
    def __init__(self, color, opacity):
        self.color = color
        self.opacity = opacity
        display.DisplayMethod.__init__(self)

    def draw(self, gfxwindow, device):
        skel = self.who().resolve(gfxwindow)
        if skel is not None:
            device.set_lineColor(self.color)
            device.set_fillColorAlpha(self.color, color.alpha(self.opacity))
            skel.elementselection.begin_reading()
            try:
                for e in skel.elementselection.retrieve():
                    device.fill_polygon(primitives.Polygon([x.position()
                                                            for x in e.nodes]))
            finally:
                skel.elementselection.end_reading()
    def getTimeStamp(self, gfxwindow):
        return max(self.timestamp,
                   self.who().resolve(gfxwindow).elementselection.timestamp)
                
                
# This object should be created via the registration, and not
# directly via the initializer, because the registration creation
# method gives it a timestamp.

elementSelectDisplay = registeredclass.Registration(
    'Selected Elements',
    display.DisplayMethod,
    SkeletonElementSelectionDisplay,
    params=[color.ColorParameter('color', color.RGBColor(0.88, 0.14, 0.07),
                                 tip="Color for the selected elements."),
            parameter.FloatRangeParameter('opacity', (0., 1., 0.01), 0.6,
                                      tip="Opacity of the selected elements.")
            ],
    ordering=2.0,
    layerordering=display.SemiPlanar,
    whoclasses=('Skeleton',),
    tip="Display the currently selected elements",
    discussion=xmlmenudump.loadFile('DISCUSSIONS/engine/reg/elementselectdisplay.xml'))
                                                

display.predefineLayer('Skeleton', '<topmost>', elementSelectDisplay())
