# -*- python -*-
# $RCSfile: meshcsdisplay.py,v $
# $Revision: 1.17 $
# $Author: langer $
# $Date: 2006/10/09 17:21:07 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from oof2.SWIG.common import ooferror
from oof2.common import color
from oof2.common import primitives
from oof2.common import registeredclass
from oof2.common.IO import automatic
from oof2.common.IO import display
from oof2.common.IO import parameter
from oof2.common.IO import selectionplaceholder
from oof2.common.IO import xmlmenudump
from oof2.engine.IO import meshcsparams
import types

# Layer for showing the mesh cross-sections.
class MeshCrossSectionDisplay(display.DisplayMethod):
    def __init__(self, cross_sections, color, linewidth):
        display.DisplayMethod.__init__(self)
        self.cross_sections = cross_sections
        self.color = color
        self.linewidth = linewidth
        
    # We might need to be redrawn when the mesh's cross-sections have
    # changed.
    def getTimeStamp(self, gfxwindow):
        mesh = self.who().resolve(gfxwindow)
        return max(self.timestamp, mesh.cross_sections.timestamp)


    def draw(self, gfxwindow, device):
        mesh = self.who().resolve(gfxwindow)
        device.set_lineColor(self.color)
        device.set_lineWidth(self.linewidth)

        if self.cross_sections==selectionplaceholder.selectionPlaceHolder:
            cstoolbox = gfxwindow.getToolboxByName('Mesh_Cross_Section')
            cs = mesh.selectedCS()
            if cs:
                device.draw_segment(primitives.Segment(cs.start, cs.end))
                
        else: # List of cs names.
            for k in self.cross_sections:
                try:
                    b = mesh.cross_sections[k]
                except KeyError:
                    pass
                else:
                    device.draw_segment(
                        primitives.Segment(b.start, b.end) )

    ## TODO: Using id(self) for hashing is not recommended in Python
    ## 2.5, since ids can be longs and hash must be an int.  Why do we
    ## really need __hash__ here?  Is this used as a key?
##    def __hash__(self): # Override hash, because we contain lists
##        return id(self)
        

                  
meshCrossSectionDisplay = registeredclass.Registration(
    'Cross Section', display.DisplayMethod,
    MeshCrossSectionDisplay,
    params=[
    meshcsparams.MeshCrossSectionSetParameter(
    'cross_sections', selectionplaceholder.selectionPlaceHolder,
    tip="Which cross-section to display?"),
    color.ColorParameter('color', value=color.gray50,
                         tip="In which color?"),
    parameter.IntRangeParameter('linewidth', (0, 10), 1,
                                tip="Thickness of the line.")],
    ordering=5.0,
    layerordering=display.SemiLinear,
    whoclasses=('Mesh',),
    tip="Determine which cross sections are displayed, and how.",
    discussion=xmlmenudump.loadFile('DISCUSSIONS/engine/reg/meshcsdisplay.xml')
    )

display.predefineLayer('Mesh', '<contourable>', meshCrossSectionDisplay())
