# -*- python -*-
# $RCSfile: microstructuredisplay.py,v $
# $Revision: 1.17 $
# $Author: langer $
# $Date: 2006/10/10 21:17:29 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

# DisplayMethod for showing the Materials assigned to a Microstructure.

from oof2.SWIG.common import coord
from oof2.SWIG.engine import material
from oof2.common import color
from oof2.common import debug
from oof2.common import registeredclass
from oof2.common.IO import display
from oof2.common.IO import xmlmenudump
import weakref



class MicrostructureMaterialDisplay(display.DisplayMethod):
    def __init__(self, no_material, no_color):
        self.no_material = no_material    # color if Material isn't assigned
        self.no_color = no_color    # color if Material has no ColorProperty
        display.DisplayMethod.__init__(self)
    def draw(self, gfxwindow, device):
        microstructure = self.who().getObject(gfxwindow)
        # The draw_image() routine requires an object with a
        # .fillstringimage() function.  We can't simply give
        # Microstructure such a function, since it can be displayed in
        # many ways, so we construct a temporary object,
        # MaterialImage, just to pass to draw_image().
        matlimage = material.MaterialImage(microstructure, self.no_material,
                                           self.no_color)
        device.draw_image(matlimage, coord.Coord(0,0), microstructure.size())
    def getTimeStamp(self, gfxwindow):
        microstructure = self.who().getObject(gfxwindow)
        return max(display.DisplayMethod.getTimeStamp(self, gfxwindow),
                   material.getMaterialTimeStamp(microstructure))

registeredclass.Registration(
    'Material',
    display.DisplayMethod,
    MicrostructureMaterialDisplay,
    ordering=0,
    layerordering=display.Planar,
    params=[
    color.ColorParameter('no_material', color.black,
                         tip="Color to use if no material has been assigned to a pixel"),
    color.ColorParameter('no_color', color.blue,
                         tip="Color to use if the assigned material has no assigned color")
    ],
    whoclasses = ('Microstructure',),
    tip="Display the color of the Material assigned to each pixel.",
    discussion=xmlmenudump.loadFile('DISCUSSIONS/engine/reg/microstructuredisplay.xml')
    )
