# -*- python -*-
# $RCSfile: nodeselectdisplay.py,v $
# $Revision: 1.21 $
# $Author: langer $
# $Date: 2006/10/06 20:41:35 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from oof2.common import color
from oof2.common import debug
from oof2.common import registeredclass
from oof2.common.IO import display
from oof2.common.IO import parameter
from oof2.common.IO import xmlmenudump

class SkeletonNodeSelectionDisplay(display.DisplayMethod):
    def __init__(self, color, size):
        self.color = color
        self.size = size
        display.DisplayMethod.__init__(self)
    def draw(self, gfxwindow, device):
        skel = self.who().resolve(gfxwindow)
        if skel is not None:
            device.set_lineColor(self.color)
            device.set_lineWidth(self.size)
            ## debug.fmsg("::before drawing dot")
            retr = skel.nodeselection.retrieve().copy()
            size = len(retr)
            for node in retr:
                device.draw_dot(node.position())
##            for n in range(size):
                ## debug.fmsg("during drawing dot")
##                device.draw_dot(retr[n].position())
            ## debug.fmsg("::after drawing dot")
    def getTimeStamp(self, gfxwindow):
        return max(self.timestamp,
                   self.who().resolve(gfxwindow).nodeselection.timestamp)

# This object should be created via the registration, and not
# directly via the initializer, becasue the registration creation
# method gives it a timestamp.

nodeSelectDisplay = registeredclass.Registration(
    'Selected Nodes',
    display.DisplayMethod,
    SkeletonNodeSelectionDisplay,
    params=[color.ColorParameter('color', color.RGBColor(0.07, 0.09, 0.96),
                                 tip="Color for the selected nodes."),
            parameter.IntRangeParameter('size', (0,10), 2,
                                        tip="Node size.")],
    ordering=2.2,
    layerordering= display.PointLike(3),
    whoclasses=('Skeleton',),
    tip="Display the currently selected nodes.",
    discussion=xmlmenudump.loadFile('DISCUSSIONS/engine/reg/nodeselectdisplay.xml')
    )

display.predefineLayer('Skeleton', '<topmost>', nodeSelectDisplay())
