// -*- C++ -*-
// $RCSfile: orientation.h,v $
// $Revision: 1.13 $
// $Author: langer $
// $Date: 2006/12/19 20:29:48 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>

#ifndef ORIENTATION_H
#define ORIENTATION_H

#include "engine/property.h"
#include "engine/eulerangle.h"
#include "engine/matvec.h"
#include <string>

class SymmMatrix3;

class OrientationProp : public Property {
private:
  EulerAngle orient;
public:
  OrientationProp(PyObject *registry, const std::string &name, 
		  const EulerAngle &ea);
  OrientationProp(PyObject *registry, const std::string &name,
		  EulerAngle *ea);
  const EulerAngle &eulerangle() const { return orient; }

  // From the comments in eulerangle.C:
  // ... The rotation matrix multiplied by a vector gives the
  // coordinates of the vector in a coordinate system that has been
  // rotated by the Euler angle.

  MATRIX_D rotate(const MATRIX_D&) const;
  VECTOR_D rotate(const VECTOR_D&) const;
  SymmMatrix3 rotate(const SymmMatrix3&) const;
  virtual bool is_symmetric(const CSubProblem*) const { return true; }
};

#endif
