# -*- python -*-
# $RCSfile: mesh_modify_test.py,v $
# $Revision: 1.6 $
# $Author: langer $
# $Date: 2007/02/10 20:57:37 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

import unittest, os
from oof2.common import primitives
Point = primitives.Point

class OOF_Mesh_Modify(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    # Since there are only a few modifiers, and they work differently,
    # just do them separately in their own individual tests.  The
    # complicated one is "Adaptive Mesh Refinement", it's saved for
    # last.  The easy ones are "Assign Material", "Remove Material",
    # and "Revert Mesh".

    def Assign_Material(self):
        from oof2.engine import skeletoncontext
        from oof2.engine import mesh
        
        OOF.File.Load.Data(filename=os.path.join(
            "mesh_data","modifiable_mesh"))

        OOF.Mesh.Modify(mesh="mesh_mod_test:skeleton:mesh",
                        modifier=AssignMaterial(
            material="other", target=MeshElementGroup(group="central")))
        
        skc=skeletoncontext.skeletonContexts["mesh_mod_test:skeleton"]
        egr=skc.elementgroups.get_group("central")
        mesh_indices = [e.meshindex for e in egr]
        mesh_obj=mesh.meshes["mesh_mod_test:skeleton:mesh"].getObject()

        ei = mesh_obj.element_iterator()
        while not ei.end():
            el = ei.element()
            mat = el.getExplicitMaterial()
            if mat:
                self.assert_(el.get_index() in mesh_indices)
            else:
                self.assert_(not el.get_index() in mesh_indices)
            ei.next()
                
        OOF.Microstructure.Delete(microstructure="mesh_mod_test")
        OOF.Material.Delete(name="soft")
        OOF.Material.Delete(name="firm")
        OOF.Material.Delete(name="other")
        

    def Remove_Material(self):
        from oof2.engine import mesh
        OOF.File.Load.Data(filename=os.path.join(
            "mesh_data","modifiable_mesh"))

        OOF.Mesh.Modify(mesh="mesh_mod_test:skeleton:mesh",
                        modifier=AssignMaterial(
            material="other", target=MeshElementGroup(group="central")))
        #
        OOF.Mesh.Modify(mesh="mesh_mod_test:skeleton:mesh",
                        modifier=RemoveMaterial(
            target=MeshElementGroup(group="central")))

        mesh_obj=mesh.meshes["mesh_mod_test:skeleton:mesh"].getObject()
        ei=mesh_obj.element_iterator()
        while not ei.end():
            el=ei.element()
            mat=el.getExplicitMaterial()
            self.assert_(not mat)
            ei.next()
        
        OOF.Microstructure.Delete(microstructure="mesh_mod_test")
        OOF.Material.Delete(name="soft")
        OOF.Material.Delete(name="firm")
        OOF.Material.Delete(name="other")


    def Adaptive_Mesh_Refinement(self):
        ## TODO: Write this test
        pass

    def Revert_Mesh(self):
        ## TODO: Write this test
        pass



# Routine to do regression-type testing on the items in this file.
# Tests must be run in the order they appear in the list.  This
# routine will stop after the first failure.

def run_tests():

    from oof2.SWIG.common import config
    if config.devel():
        print "Devel mode detected, modify testing skipped."
        return 0
    
    modify_set = [
        OOF_Mesh_Modify("Assign_Material"),
        OOF_Mesh_Modify("Remove_Material"),
##        OOF_Mesh_Modify("Adaptive_Mesh_Refinement"),
##        OOF_Mesh_Modify("Revert_Mesh")
        ]
    
    test_set = modify_set

    logan = unittest.TextTestRunner()
    for t in test_set:
        print "\n *** Running test: %s\n" % t.id()
        res = logan.run(t)
        if not res.wasSuccessful():
            return 0
    return 1


###################################################################
# The code below this line should be common to all testing files. #
###################################################################

if __name__=="__main__":
    # If directly run, then start oof, and run the local tests, then quit.
    import sys
    try:
        from oof2.common import oof
    except ImportError:
        print "OOF is not correctly installed on this system."
        sys.exit(4)
    sys.argv.append("--text")
    sys.argv.append("--quiet")
    sys.argv.append("--seed=17")
    oof.run(no_interp=1)

    success = run_tests()

    OOF.File.Quit()
    
    if success:
        print "All tests passed."
    else:
        print "Test failure."
